using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Shared.Models;

public class Face : Properties.IFace
{

    protected DateTime _DateTime;
    protected List<FaceDistance> _FaceDistances;
    protected FaceEncoding? _FaceEncoding;
    protected Dictionary<Stateless.FacePart, FacePoint[]>? _FaceParts;
    protected List<int[]>? _FaceNumbers;
    protected readonly OutputResolution? _OutputResolution;
    protected Location? _Location;
    protected readonly int? _LocationIndex;
    protected readonly string _RelativePath;
    protected List<object> _Tags;
    public DateTime DateTime => _DateTime;
    public List<FaceDistance> FaceDistances => _FaceDistances;
    public FaceEncoding? FaceEncoding => _FaceEncoding;
    public Dictionary<Stateless.FacePart, FacePoint[]>? FaceParts => _FaceParts;
    public List<int[]>? FaceNumbers => _FaceNumbers;
    public Location? Location => _Location;
    public int? LocationIndex => _LocationIndex;
    public OutputResolution? OutputResolution => _OutputResolution;
    public string RelativePath => _RelativePath;
    public List<object> Tags => _Tags;

    [JsonConstructor]
    public Face(DateTime dateTime, List<FaceDistance> faceDistances, FaceEncoding? faceEncoding, Dictionary<Stateless.FacePart, FacePoint[]>? faceParts, List<int[]>? faceNumbers, Location? location, int? locationIndex, OutputResolution? outputResolution, string relativePath)
    {
        if (faceDistances is null)
            faceDistances = new();
        _DateTime = dateTime;
        _FaceDistances = faceDistances;
        _FaceEncoding = faceEncoding;
        _FaceParts = faceParts;
        _FaceNumbers = faceNumbers;
        _Location = location;
        _LocationIndex = locationIndex;
        _OutputResolution = outputResolution;
        _RelativePath = relativePath;
        _Tags = new();
    }

    public Face() :
        this(DateTime.MinValue, new(), null, null, null, null, null, null, string.Empty)
    { }

    public Face(Location location) :
        this(DateTime.MinValue, new(), null, null, null, location, null, null, string.Empty)
    { }

    public Face(int facesCount, Face face) :
        this(face.DateTime, new(), face.FaceEncoding, face.FaceParts, null, face.Location, face.LocationIndex, face.OutputResolution, face.RelativePath)
    {
        if (face.Location?.Confidence is not null && face.OutputResolution is not null)
            _Location = new(face.Location.Confidence, face.OutputResolution.Height, face.Location, face.OutputResolution.Width, facesCount);
    }

    public Face(int outputResolutionWidth, int outputResolutionHeight, int outputResolutionOrientation, Face face) :
        this(face.DateTime, new(), face.FaceEncoding, face.FaceParts, null, face.Location, face.LocationIndex, null, face.RelativePath)
    {
        if (outputResolutionHeight > 0)
            _OutputResolution = new(outputResolutionHeight, outputResolutionOrientation, outputResolutionWidth);
    }

    public Face(Property property, int outputResolutionWidth, int outputResolutionHeight, int outputResolutionOrientation, string relativePath, int? i, Location? location) :
        this(DateTime.MinValue, new(), null, null, null, location, i, null, relativePath)
    {
        DateTime?[] dateTimes;
        _OutputResolution = new(outputResolutionHeight, outputResolutionOrientation, outputResolutionWidth);
        dateTimes = new DateTime?[] { property.CreationTime, property.LastWriteTime, property.DateTime, property.DateTimeDigitized, property.DateTimeOriginal, property.GPSDateStamp };
        _DateTime = (from l in dateTimes where l.HasValue select l.Value).Min();
    }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    public void SetFaceEncoding(FaceEncoding faceEncoding) => _FaceEncoding = faceEncoding;

    public void SetFaceParts(Dictionary<Stateless.FacePart, FacePoint[]> faceParts) => _FaceParts = faceParts;

    public void SetFaceNumbers(List<int[]> faceNumbers) => _FaceNumbers = faceNumbers;

}